
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;

/**
 * User: Mart Karu
 * Date: 22.04.2003
 * Time: 11:27:08
 */
public class PildiPaneel extends JPanel
{
    BufferedImage pilt = null;

    public PildiPaneel()
    {
    }

    public PildiPaneel(BufferedImage pilt)
    {
        setPilt(pilt);
    }

    /**
     * Nidatava pildi seadmine
     * @param pilt uus pilt
     */
    public void setPilt(BufferedImage pilt)
    {
        this.pilt = pilt;
        setSize(pilt.getWidth(), pilt.getHeight());
        repaint();
    }

    /**
     * Uuendab ekraani
     * @param g
     */
    public void paint(Graphics g)
    {
        super.paint(g);
        if (pilt != null)
            g.drawImage(pilt, 0, 0, this);
    }

    /**
     * Tagastab pildi vasaku lemise piksli
     * @return vasak lemine piksel
     */
    public int getYlemineVasakPiksel()
    {
        if (pilt != null)
        {
            int piksel = pilt.getRGB(0, 0);
            return piksel;
        }
        return 0;
    }

    /**
     * Filtreerib pildi mustvalgeks
     */
    public void teeMustValgeks()
    {
        BufferedImage abiPilt = new BufferedImage(pilt.getWidth(), pilt.getHeight(), pilt.getType());
        float andmed[][] = new float[][]{
            {(float) 0.3, (float) 0.3, (float) 0.3},
            {(float) 0.3, (float) 0.3, (float) 0.3},
            {(float) 0.3, (float) 0.3, (float) 0.3}
        };
        BandCombineOp bco = new BandCombineOp(andmed, null);
        bco.filter(pilt.getData(), abiPilt.getRaster());
        pilt = abiPilt;
        repaint();
    }

    /**
     * Salvestab pildi faili
     * @param f salvestatav fail
     */
    public void salvestaFaili(File f)
    {
        FileOutputStream fos;
        try
        {
            DataOutputStream dos = new DataOutputStream(new FileOutputStream(f));
            dos.writeInt(pilt.getWidth());
            dos.writeInt(pilt.getHeight());
            for (int i = 0; i < pilt.getWidth(); i++)
                for (int j = 0; j < pilt.getHeight(); j++)
                    dos.writeInt(pilt.getRGB(i, j));
            dos.close();
        } catch (FileNotFoundException e)
        {
            e.printStackTrace();
        } catch (IOException e)
        {
            e.printStackTrace();
        }

    }

    /**
     * Loeb failist pildi
     * @param f fail lugemiseks
     */
    public void avaFailist(File f)
    {
        FileInputStream fis;
        try
        {
            DataInputStream dis = new DataInputStream(new FileInputStream(f));
            int uusLaius = dis.readInt();
            int uusKorgus = dis.readInt();

            BufferedImage uusPilt = new BufferedImage(uusLaius, uusKorgus, BufferedImage.TYPE_INT_RGB);

            for (int i = 0; i < uusLaius; i++)
                for (int j = 0; j < uusKorgus; j++)
                    uusPilt.setRGB(i, j, dis.readInt());

            dis.close();
            setPilt(uusPilt);

        } catch (FileNotFoundException e)
        {
            e.printStackTrace();
        } catch (IOException e)
        {
            e.printStackTrace();
        }
    }
}
